package de.hdm_stuttgart.mi.sd1.test.aufgabe2;


import de.hdm_stuttgart.mi.sd1.aufgabe2.MaexleWurf;
import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;

import java.util.Arrays;

/**
 * Teste Ordnung von Würfelergebnissen beim »Mäxchen« Spiel.
 *
 */

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
@SuppressWarnings({"UnusedDeclaration"})
public class Test_MaexchenVergleich extends ExaminationTestDefaults {

  static private final int countNichtSonderfälle = (5 * 6) / 2 - 1; // (3|1), (3|2), ...

  private static final MaexleWurf[]
          nichtSonderFaelle1 = new MaexleWurf[countNichtSonderfälle],
          nichtSonderFaelle2 = new MaexleWurf[countNichtSonderfälle],

  paschFaelle = new MaexleWurf[6],
          maexchenFaelle = new MaexleWurf[2];

  static {
    int index = 0;
    for (int i = 3; i <= 6; i++) { // 5 + 4 + 3 + 2 + 1 == countNichtSonderfälle
      for (int j = 1; j < i; j++) {
        nichtSonderFaelle1[index] = new MaexleWurf(i, j);
        nichtSonderFaelle2[index] = new MaexleWurf(j, i);
        index++;
      }
    }
    for (int i = 1; i <= 6; i++) {
      paschFaelle[i - 1] = new MaexleWurf(i, i);
    }
    maexchenFaelle[0] = new MaexleWurf(1, 2);
    maexchenFaelle[1] = new MaexleWurf(2, 1);
  }

  /**
   * toString()
   */
  @Test @Marking(points = 2) public void test_80() {
    Assert.assertEquals("(3|1)", new MaexleWurf(1, 3).toString());
    Assert.assertEquals("(3|1)", new MaexleWurf(3, 1).toString());
  }

  /**
   * Gleichheit nicht-Sonderfälle
   */
  @Test @Marking(points = 4) public void test_100() {
    for (int i = 0; i < countNichtSonderfälle; i++) {
      Assert.assertEquals("i = " + i, 0, nichtSonderFaelle1[i].compareTo(nichtSonderFaelle1[i]));
      Assert.assertEquals("i = " + i, 0, nichtSonderFaelle2[i].compareTo(nichtSonderFaelle2[i]));

      Assert.assertEquals("i = " + i, 0, nichtSonderFaelle1[i].compareTo(nichtSonderFaelle2[i]));
      Assert.assertEquals("i = " + i, 0, nichtSonderFaelle2[i].compareTo(nichtSonderFaelle1[i]));

      Assert.assertTrue("i = " + i, 0 < nichtSonderFaelle1[i].compareTo(new MaexleWurf(i , i)));
      Assert.assertTrue("i = " + i, 0 < nichtSonderFaelle1[i].compareTo(new MaexleWurf(1 , 2)));
      Assert.assertTrue("i = " + i, 0 < nichtSonderFaelle1[i].compareTo(new MaexleWurf(2 , 1)));
    }

  }

  /**
   * Gleichheit Pasch
   */
  @Test @Marking(points = 2) public void test_120() {
    for (int i = 1; i <= 6; i++) {
      Assert.assertEquals("i = " + i, 0, paschFaelle[i - 1].compareTo(paschFaelle[i - 1]));
      Assert.assertTrue("i = " + i, 0 < nichtSonderFaelle1[i].compareTo(paschFaelle[i - 1]));
    }
  }

  /**
   * Gleichheit Mäxchen
   */
  @Test @Marking(points = 2) public void test_140() {
    Assert.assertEquals(0, new MaexleWurf(1, 2).compareTo(new MaexleWurf(1, 2)));
    Assert.assertEquals(0, new MaexleWurf(1, 2).compareTo(new MaexleWurf(2, 1)));
    Assert.assertEquals(0, new MaexleWurf(2, 1).compareTo(new MaexleWurf(1, 2)));
    Assert.assertEquals(0, new MaexleWurf(2, 1).compareTo(new MaexleWurf(2, 1)));
  }

  /**
   * Ungleichheit Normalfälle untereinander
   */
  @Test @Marking(points = 2) public void test_220() {
    for (int i = 0; i < countNichtSonderfälle; i++) {
      for (int j = i + 1; j < countNichtSonderfälle; j++) {
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle1[i].compareTo(nichtSonderFaelle1[j]));
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle1[i].compareTo(nichtSonderFaelle2[j]));
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle2[i].compareTo(nichtSonderFaelle1[j]));
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle2[i].compareTo(nichtSonderFaelle2[j]));

        Assert.assertTrue("i = " + i + ", j = " + j, nichtSonderFaelle1[j].compareTo(nichtSonderFaelle1[i]) < 0);
        Assert.assertTrue("i = " + i + ", j = " + j, nichtSonderFaelle1[j].compareTo(nichtSonderFaelle2[i]) < 0);
        Assert.assertTrue("i = " + i + ", j = " + j, nichtSonderFaelle2[j].compareTo(nichtSonderFaelle1[i]) < 0);
        Assert.assertTrue("i = " + i + ", j = " + j, nichtSonderFaelle2[j].compareTo(nichtSonderFaelle2[i]) < 0);
      }
    }
  }
  /**
   * Ungleichheit Pasch - Normalfall
   */
  @Test @Marking(points = 4) public void test_240() {
    for (int i = 0; i < countNichtSonderfälle; i++) {
      for (int j = 0; j < paschFaelle.length;j++) {

        final MaexleWurf p = paschFaelle[j];
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle1[i].compareTo(p));
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle2[i].compareTo(p));

        Assert.assertTrue("i = " + i + ", j = " + j, p.compareTo(nichtSonderFaelle1[i]) < 0);
        Assert.assertTrue("i = " + i + ", j = " + j, p.compareTo(nichtSonderFaelle2[i]) < 0);
      }
    }
  }

  /**
   * Ungleichheit Mäxchen - Normalfall
   */
  @Test @Marking(points = 2) public void test_260() {
    for (int i = 0; i < nichtSonderFaelle1.length; i++) {

      for (int j = 0; j < maexchenFaelle.length; j++) {

        final MaexleWurf m = maexchenFaelle[j];

        Assert.assertTrue("i = " + i + ", j = " + j, m.compareTo(nichtSonderFaelle1[i]) < 0);
        Assert.assertTrue("i = " + i + ", j = " + j, m.compareTo(nichtSonderFaelle2[i]) < 0);

        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle1[i].compareTo(m));
        Assert.assertTrue("i = " + i + ", j = " + j, 0 < nichtSonderFaelle2[i].compareTo(m));
      }
    }
  }

  /**
   * Ungleichheit Mäxchen - Pasch
   */
  @Test @Marking(points = 2) public void test_280() {
    for (int i = 0; i < maexchenFaelle.length; i++) {
      final MaexleWurf m = maexchenFaelle[i];
      for (int j = 0; j < paschFaelle.length; j++) {
        final MaexleWurf p = paschFaelle[j];

        Assert.assertTrue(0 < p.compareTo(m));
        Assert.assertTrue(m.compareTo(p) < 0);
      }
    }
  }
}
